# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Tests for template context processors."""

from __future__ import absolute_import, print_function

from flask import Flask

from invenio_formatter import InvenioFormatter


def test_views_badge_svg(app):
    """Test context processor badge generating a SVG."""
    with app.app_context():
        with app.test_client() as client:
            response = client.get('/badge/DOI/value.svg')
            response_data = response.get_data(as_text=True).replace(
                    '\n', '').replace(' ', '')
            assert 'fill-opacity=".3">DOI</text>' in response_data
            assert 'y="14">DOI</text>' in response_data
            assert 'fill-opacity=".3">value</text>' in response_data
            assert 'y="14">value</text>' in response_data

            # Unallowed title
            assert client.get('/badge/invalid/value.svg').status_code == 404


def test_views_badge_svg_mapping():
    """Test context processor badge generating a SVG."""
    """Flask application fixture."""
    app = Flask('testapp')
    app.config.update(
        TESTING=True,
        FORMATTER_BADGES_ALLOWED_TITLES=['test', 'TEST'],
        FORMATTER_BADGES_TITLE_MAPPING={'test': 'TEST'},
    )
    InvenioFormatter(app)

    with app.app_context():
        with app.test_client() as client:
            response = client.get('/badge/test/value.svg')
            response_data = response.get_data(as_text=True).replace(
                    '\n', '').replace(' ', '')
            assert 'fill-opacity=".3">TEST</text>' in response_data

            response = client.get('/badge/TEST/value.svg')
            response_data = response.get_data(as_text=True).replace(
                    '\n', '').replace(' ', '')
            assert 'fill-opacity=".3">TEST</text>' in response_data


def test_views_badge_png(app):
    """Test context processor badge generating a SVG."""
    with app.app_context():
        with app.test_client() as client:
            response = client.get('/badge/DOI/value.png')
            assert b'\x89PNG\r\n' in response.data



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Tests for Jinja2 filters."""

from __future__ import absolute_import, print_function

from datetime import date, datetime

import arrow
import pytest
from arrow.parser import ParserError
from flask import render_template_string


def test_from_isodate(app):
    """Test from_isodate filter."""
    with app.test_request_context():
        assert render_template_string(
            "{{ 'yes' if dt|from_isodate < now else 'no'}}",
            dt='2002-01-01', now=date.today()) == 'yes'
        assert render_template_string("{{ dt|from_isodate }}", dt='') == 'None'
        assert render_template_string(
            "{{ dt|from_isodate }}", dt=datetime(2002, 1, 1, 1, 1)) == \
            '2002-01-01'
        pytest.raises(
            TypeError, render_template_string,
            "{{ 'yes' if dt < now else 'no'}}",
            dt='2002-01-01', now=date.today())
        pytest.raises(
            ParserError,
            render_template_string, "{{ dt|from_isodate }}", dt='abcd-01-01')
        pytest.raises(
            ParserError, render_template_string,
            "{{ dt|from_isodate(strict=True) }}", dt='')

        # Test pre-1900 centuries.
        assert render_template_string(
            "{{ '0001-01-01'|from_isodate > '1500-01-01'|from_isodate }}") == \
            "False"


def test_from_isodatetime(app):
    """Test from_isodate filter."""
    with app.test_request_context():
        assert render_template_string(
            "{{ 'yes' if dt|from_isodatetime < now else 'no'}}",
            dt='2002-01-01T00:01:00', now=arrow.now()) == 'yes'

        assert render_template_string("{{ dt|from_isodatetime }}", dt='') \
            == 'None'
        assert render_template_string(
            "{{ dt|from_isodatetime }}", dt=datetime(2002, 1, 1, 1, 1)) == \
            '2002-01-01 01:01:00+00:00'
        pytest.raises(
            TypeError, render_template_string,
            "{{ 'yes' if dt < now else 'no'}}",
            dt='2002-01-01T00:01', now=arrow.now())
        pytest.raises(
            ParserError,
            render_template_string, "{{ dt|from_isodatetime }}",
            dt='abcd-01-01T00:00:00')
        pytest.raises(
            ParserError, render_template_string,
            "{{ dt|from_isodatetime(strict=True) }}", dt='')

        # Test pre-1900 centuries.
        assert render_template_string(
            "{{ '0001-01-01T00:00:00'|from_isodatetime"
            " > '1500-01-01'|from_isodatetime }}") == \
            "False"



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015, 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Jinja utilities for Invenio."""

from __future__ import absolute_import, print_function

from pkg_resources import DistributionNotFound, get_distribution

from . import config
from .filters.datetime import from_isodate, from_isodatetime
from .views import create_badge_blueprint


class InvenioFormatter(object):
    """Invenio-Formatter extension."""

    def __init__(self, app=None):
        """Extension initialization."""
        if app:
            self.init_app(app)

    def init_app(self, app):
        """Flask application initialization."""
        self.init_config(app)

        # Install datetime helpers.
        app.jinja_env.filters.update(
            from_isodate=from_isodate,
            from_isodatetime=from_isodatetime,
        )

        if app.config['FORMATTER_BADGES_ENABLE']:
            from invenio_formatter.context_processors.badges import \
                badges_processor

            # Registration of context processors.
            app.context_processor(badges_processor)
            # Register blueprint.
            app.register_blueprint(create_badge_blueprint(
                app.config['FORMATTER_BADGES_ALLOWED_TITLES']))

        app.extensions['invenio-formatter'] = self

    def init_config(self, app):
        """Initialize configuration."""
        try:
            get_distribution('CairoSVG')
            has_cairo = True
        except DistributionNotFound:
            has_cairo = False

        app.config.setdefault('FORMATTER_BADGES_ENABLE', has_cairo)

        for attr in dir(config):
            if attr.startswith('FORMATTER_'):
                app.config.setdefault(attr, getattr(config, attr))



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Badges context processor."""

from __future__ import absolute_import, print_function

import cairosvg

from base64 import b64encode
from PIL import Image, ImageDraw, ImageFont


def get_text_length(*args):
    """Measure the size of string rendered with a TTF no-nomospaced fonts."""
    txt = Image.new('RGBA', (16, 16), (255, 255, 255, 0))
    d = ImageDraw.Draw(txt)
    font = ImageFont.truetype("DejaVuSans", 11)
    result = ()
    for value in args:
        result = result + (d.textsize(value, font=font)[0], )
    return result


def generate_badge_svg(title, value, color='#007ec6'):
    """Generate the SVG."""
    (title_length, value_length) = get_text_length(title, value)
    return '''<svg xmlns="http://www.w3.org/2000/svg"
     width="{width}" height="20">
        <linearGradient id="b" x2="0" y2="100%">
            <stop offset="0" stop-color="#bbb" stop-opacity=".1"/>
            <stop offset="1" stop-opacity=".1"/>
        </linearGradient>
        <mask id="a" width="{width}" height="20">
            <rect width="{width}" height="20" rx="3"
            fill="#fff"/>
        </mask>
        <g mask="url(#a)">
            <path fill="#555" d="M0 0h{title_width}v20H0z" />
            <path fill="{color}"
            d="M{title_width} 0h{value_width}v20H{title_width}z"
            />
            <path fill="url(#b)" d="M0 0h{width}v20H0z" />
        </g>
        <g fill="#fff" text-anchor="middle" font-family="DejaVu Sans,
        Verdana,Geneva,sans-serif" font-size="11">
            <text x="{title_position}" y="15" fill="#010101"
            fill-opacity=".3">
                {title}
            </text>
            <text x="{title_position}" y="14">
                {title}
            </text>
            <text x="{value_position}"
            y="15" fill="#010101" fill-opacity=".3">
                {value}
            </text>
            <text x="{value_position}" y="14">
                {value}
            </text>
        </g>
    </svg>'''.format(
        title_width=title_length + 11,
        value_width=value_length + 11,
        width=title_length + value_length + 22,
        title_position=title_length/2 + 6,
        value_position=title_length + value_length/2 + 16,
        title=title,
        value=value,
        color=color,
    )


def generate_badge_png(title, value, color='#007ec6'):
    """Generate the badge in PNG format."""
    badge = generate_badge_svg(title, value, color)
    return cairosvg.svg2png(badge)


def badges_processor():
    """Context processor for badges."""
    def badge_svg(title, value, color='#007ec6'):
        """Context processor function to generate SVG badges."""
        return generate_badge_svg(title, value, color)

    def badge_png(title, value, color='#007ec6'):
        """Context processor function to generate SVG badges."""
        png = generate_badge_png(title, value, color)
        png_base64 = b64encode(png)
        return 'data:image/png;base64,{0}'.format(png_base64)

    return dict(
            badge_svg=badge_svg,
            badge_png=badge_png
        )



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Tests for Jinja2 filters."""

from __future__ import absolute_import, print_function

from datetime import date

from flask import render_template_string


def test_meta_twittercard(app):
    """Test macro meta_twittercard."""
    template = r"""
    {% from "invenio_formatter/macros/meta.html" import meta_twittercard %}
    {{meta_twittercard("TITLE", "DESC", card="CARD", site="SITE")}}
    """
    with app.test_request_context():
        html = render_template_string(template)
        assert '<meta name="twitter:card" content="CARD" />' in html
        assert '<meta name="twitter:site" content="SITE" />' in html
        assert '<meta name="twitter:title" content="TITLE" />' in html
        assert '<meta name="twitter:description" content="DESC" />' in html


def test_meta_opengraph(app):
    """Test macro meta_twittercard."""
    template = r"""
    {% from "invenio_formatter/macros/meta.html" import meta_opengraph %}
    {{meta_opengraph("TITLE", "DESC",
                     url="http://example.org", site_name="SITE")}}
    """
    with app.test_request_context():
        html = render_template_string(template)
        assert '<meta property="og:url" content="http://example.org" />' \
            in html
        assert '<meta property="og:site_name" content="SITE" />' in html
        assert '<meta property="og:title" content="TITLE" />' in html
        assert '<meta property="og:description" content="DESC" />' in html


def test_meta_highwire(app):
    """Test macro meta_twittercard."""
    template = r"""
    {% from "invenio_formatter/macros/meta.html" import meta_highwire %}
    {{meta_highwire(
        "TITLE", "DESC", authors=["Doe, John", "Smith, Joe"],
        publication_date=today)}}
    """
    with app.test_request_context():
        html = render_template_string(template, today=date(2002, 1, 1))
        assert '<meta name="citation_publication_date"' \
               ' content="2002/01/01" />' \
               in html
        assert '<meta name="citation_author"' \
               ' content="Doe, John" />' \
               in html
        assert '<meta name="citation_author"' \
               ' content="Smith, Joe" />' \
               in html


def test_meta_badges_formats_list(app):
    """Test macro badges_formats_list."""
    template = r"""
    {% from "invenio_formatter/macros/badges.html" import badges_formats_list
     %}
    {{ badges_formats_list('image.svg', 'link') }}
    """
    with app.test_request_context():
        html = render_template_string(template, today=date(2002, 1, 1))
        assert '<pre>&lt;a href="link"&gt;&lt;img src="image.svg"' in html
        assert '<pre>.. image:: image.svg' in html
        assert '<pre>[![DOI](image.svg)](link)' in html



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""View method for Invenio-Formatter."""

from __future__ import absolute_import, print_function

from flask import Blueprint, Response, current_app


def create_badge_blueprint(allowed_types):
    """Create the badge blueprint."""
    from invenio_formatter.context_processors.badges import \
        generate_badge_png, generate_badge_svg

    blueprint = Blueprint(
        'invenio_formatter_badges',
        __name__,
        template_folder='templates',
    )

    @blueprint.route(
        '/badge/<any({0}):title>/<path:value>.<any(svg, png):ext>'.format(
            ', '.join(allowed_types)))
    def badge(title, value, ext='svg'):
        """Generate a badge response."""
        if ext == 'svg':
            generator = generate_badge_svg
            mimetype = 'image/svg+xml'
        elif ext == 'png':
            generator = generate_badge_png
            mimetype = 'image/png'
        return Response(
            generator(
                current_app.config['FORMATTER_BADGES_TITLE_MAPPING'].get(
                    title, title),
                value),
            mimetype=mimetype)

    return blueprint



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Tests for template context processors."""

from __future__ import absolute_import, print_function

from flask import render_template_string


def test_context_processor_badge_svg(app):
    """Test context processor badge generating a SVG."""
    template = r"""
    {{ badge_svg('DOI','10.1234/zenodo.12345') }}
    """
    with app.test_request_context():
        html = render_template_string(template)
        html = html.replace('\n', '').replace(' ', '')
        assert 'fill-opacity=".3">DOI</text>' in html
        assert 'y="14">DOI</text>' in html
        assert 'fill-opacity=".3">10.1234/zenodo.12345</text>' in html
        assert 'y="14">10.1234/zenodo.12345</text>' in html


def test_context_processor_badge_png(app):
    """Test context processor badge generating a PNG."""
    template = r"""
    {{ badge_png('this_is_the_title','this_is_the_value') }}
    """
    with app.test_request_context():
        html = render_template_string(template)
        assert 'data:image/png;base64,' in html
